SuperStrict


Rem
bbdoc: Data structures/Hash tables
End Rem
Module pine.HashTable

ModuleInfo "Version: 1.00"
ModuleInfo "Author: Sophie Kirschner : meapineapple@gmail.com"
ModuleInfo "License: Creative Commons Attribution-ShareAlike 3.0 Unported (CC BY-SA 3.0)"
ModuleInfo "Modserver: pine"


Import brl.linkedlist


	Rem
	bbdoc: Create a new hash table
	returns: A new hash table
	about: @hashfunction should be a function that takes a string as an argument and returns an integer. For best results, the hash function's distribution of keys should be as close to uniform as possible.
	End Rem
Function CreateHash:HashTable(size:Int,hashfunction:Int(str:String)=dbgHashKey)
	Return HashTable.Create(size,hashfunction)
End Function
	Rem
	bbdoc: Copy a hash table
	returns: A new hash table containing all the same key and value pairs as the previous and possessing the same characteristics
	End Rem
Function CopyHash:HashTable(hash:HashTable)
	Return hash.copy()
End Function
	Rem
	bbdoc: Clear the contents of a hash table
	End Rem
Function ClearHash(hash:HashTable)
	hash.clear
End Function
	Rem 
	bbdoc: Determine the number of occupied buckets in the hash table
	returns: The number of buckets containing at least one node
	End Rem
Function HashFilled:Int(hash:HashTable)
	Return hash.checkfill()
End Function
	Rem
	bbdoc: Resize a hash table
	about: Creates a new hash table of the same size, inserts all the values inside this hash table into it, then makes the new hash table become this hash table
	End Rem
Function ResizeHash(hash:HashTable,size:Int)
	hash.resize size
End Function
	Rem
	bbdoc: Count the number of elements in the hash table
	returns: The number of nodes
	End Rem
Function CountHash:Int(hash:HashTable)
	Return hash.count()
End Function
	Rem
	bbdoc: Find the most number of elements in any bucket
	returns: The maximum number of nodes in a single bucket
	End Rem
Function HashDepth:Int(hash:HashTable)
	Return hash.maxbucketdepth()
End Function
	Rem
	bbdoc: Find the bucket in the hash table with the greatest number of elements
	returns: The index of the bucket in the hash table with the greatest depth
	about: When more than one bucket is tied for the greatest depth, the one with the lowest index will be returned
	End Rem
Function HashDeepest:Int(hash:HashTable)
	Return hash.maxbucket()
End Function
	Rem
	bbdoc: Find the bucket in the hash table with the greatest number of elements
	returns: The TList which is the bucket in the hash table with the greatest depth, contains values
	End Rem
Function HashDeepestBucket:TList(hash:HashTable)
	Return hash.bucket(hash.maxbucket())
End Function
	Rem
	bbdoc: Find the bucket in the hash table with the greatest number of elements
	returns: The TList which is the bucket in the hash table with the greatest depth, contains nodes
	End Rem
Function HashDeepestNodeBucket:TList(hash:HashTable)
	Return hash.data[hash.maxbucket()]
End Function
	Rem
	bbdoc: Find the bucket of the hash table at the specified index
	returns: The TList which serves as the bucket at that index, contains values
	End Rem
Function HashBucket:TList(hash:HashTable,index:Int)
	Return hash.bucket(index)
End Function
	Rem
	bbdoc: Find the bucket of the hash table at the specified index
	returns: The TList which serves as the bucket at that index, contains nodes
	End Rem
Function HashNodeBucket:TList(hash:HashTable,index:Int)
	Return hash.data[index]
End Function
	Rem
	bbdoc: Get the size of a hash table
	returns: The size of the hash table
	End Rem
Function HashSize:Int(hash:HashTable)
	Return hash.size()
End Function
	Rem
	bbdoc: Insert a new key and value pair into a hash table
	returns: The node in the hash table containing this new data
	End Rem
Function HashInsert:HashNode(hash:HashTable,key:String,value:Object)
	Return hash.insert(key,value)
End Function
	Rem
	bbdoc: Insert a node into a hash table
	End Rem
Function HashInsertNode(hash:HashTable,node:HashNode)
	hash.insertnode(node)
End Function
	Rem
	bbdoc: Find the first value corresponding to some key in the hash table
	returns: The first value associated with the given key
	End Rem
Function HashFind:Object(hash:HashTable,key:String)
	Return hash.find(key)
End Function
	Rem
	bbdoc: Find the first node with some key in the hash table
	returns: The first encountered node with the given key
	End Rem
Function HashFindNode:HashNode(hash:HashTable,key:String)
	Return hash.findnode(key)
End Function
	Rem
	bbdoc: Find all the values corresponding to some key in the hash table
	returns: A list of the values associated with the given key
	End Rem
Function HashFindAll:TList(hash:HashTable,key:String)
	Return hash.findall(key)
End Function
	Rem
	bbdoc: Find all the nodes with some key in the hash table
	returns: A list of the nodes with the given key
	End Rem
Function HashFindAllNodes:TList(hash:HashTable,key:String)
	Return hash.findallnodes(key)
End Function
	Rem
	bbdoc: Remove the first node with a given key
	returns: The node that was removed
	End Rem
Function HashRemove:HashNode(hash:HashTable,key:String)
	Return hash.remove(key)
End Function
	Rem
	bbdoc: Remove all the nodes with a given key
	returns: The number of nodes that were removed
	End Rem
Function HashRemoveAll:Int(hash:HashTable,key:String)
	Return hash.removeall(key)
End Function
	Rem
	bbdoc: Remove all the nodes with a given key
	returns: A list containing all the nodes which were removed
	End Rem
Function HashRemoveAllNodes:TList(hash:HashTable,key:String)
	Return hash.removeallnodes(key)
End Function
	Rem
	bbdoc: Returns an iterator object to that can be used with #EachIn
	returns: An iterator object
	about: Iterates through the objects in the hash table
	End Rem
Function HashObjects:HashEnum(hash:HashTable)
	Return hash.ObjectEnumerator()
End Function
	Rem
	bbdoc: Returns an iterator object to that can be used with #EachIn
	returns: An iterator object
	about: Iterates through the keys in the hash table, if a key appears in the hash table more than once then it will go to the iterator more than once.
	End Rem
Function HashKeys:HashEnum(hash:HashTable)
	Return hash.KeyEnumerator()
End Function
	Rem
	bbdoc: Returns an iterator object to that can be used with #EachIn
	returns: An iterator object
	about: Iterates through the nodes in the hash table
	End Rem
Function HashNodes:HashNodeEnum(hash:HashTable)
	Return hash.NodeEnumerator()
End Function
	Rem
	bbdoc: Gets node key
	returns: The node's key
	End Rem
Function HashNodeKey:String(node:HashNode)
	Return node.key
End Function
	Rem
	bbdoc: Gets node value
	returns: The node's value
	End Rem
Function HashNodeValue:Object(node:HashNode)
	Return node.data
End Function
	Rem
	bbdoc: Create a new hash table node
	returns: The new node
	End Rem
Function CreateHashNode:HashNode(key:String,value:Object)
	Return HashNode.Create(key,value)
End Function
	Rem
	bbdoc: Set whether values are brought to the front of a bucket for quicker finding in the future whenever they are accessed
	End Rem
Function HashSetBringToFront(hash:HashTable,bringtofront:Int)
	hash.autobringtofront=bringtofront
End Function
	Rem
	bbdoc: Get whether values are brought to the front of a bucket for quicker finding in the future whenever they are accessed
	returns: The boolean value
	End Rem
Function HashGetBringToFront:Int(hash:HashTable)
	Return hash.autobringtofront
End Function


	Rem
	bbdoc: A hash table object
	End Rem
Type HashTable
	Field data:TList[]
	Field hashfunc%(key$)
	Field autobringtofront%=True
	Rem
	bbdoc: Set table size
	about: Will delete any existing elements in the table
	End Rem
	Method setsize(size%)
		data=New TList[size]
	End Method
	Rem
	bbdoc: Determine the number of buckets occupied by at least one element
	returns: The number of buckets occupied by at least one element
	End Rem
	Method checkfill%()
		'return the number of buckets occupied
		Local total%=0
		For Local x%=0 To data.length-1
			If data[x] And Not data[x].isempty() Then total:+1
		Next
		Return total
	End Method
	Rem
	bbdoc: Set table size
	about: Will retain the elements which already exist in the table
	End Rem
	Method resize(size%)
		Local temp:TList=CreateList()
		For Local x%=0 To data.length-1
			If data[x] Then temp.addlast data[x]
		Next
		setsize(size)
		For Local t:TList=EachIn temp
			For Local n:HashNode=EachIn t
				insertnode n
			Next
		Next
	End Method
	Rem
	bbdoc: Create a copy of the hash table
	returns: A new hash table with the same properties and containing the same key and value pairs as this one
	End Rem
	Method copy:HashTable()
		Local n:HashTable=HashTable.Create(data.length,hashfunc)
		n.autobringtofront=autobringtofront
		For Local x%=0 To data.length-1
			If data[x] Then n.data[x]=data[x].copy()
		Next
		Return n
	End Method
	Rem
	bbdoc: Clear the contents of the hash table
	End Rem
	Method clear()
		For Local x%=0 To data.length-1
			If data[x] Then data[x].clear
		Next
	End Method
	Rem
	bbdoc: Determine the number of elements in the hash table
	returns: The number of elements in the hash table
	End Rem
	Method count%()
		Local sum%=0
		For Local x%=0 To data.length-1
			If data[x] Then sum:+data[x].count()
		Next
		Return sum
	End Method
	Rem
	bbdoc: Find the greatest number of elements contained in a single bucket
	returns: The greatest number of elements in a single bucket
	End Rem
	Method maxbucketdepth%()
		Local ret%=0
		For Local x%=0 To data.length-1
			Local c%=0
			If data[x] Then c=data[x].count()
			If c>ret Then ret=c
		Next
		Return ret
	End Method
	Rem
	bbdoc: Find the bucket with the greatest number of elements
	returns: The index of the bucket which contains the greatest number of elements
	End Rem
	Method maxbucket%()
		'return the index of the bucket with the greatest depth
		Local ret%=0,bucket%=0
		For Local x%=0 To data.length-1
			Local c%=0
			If data[x] Then c=data[x].count()
			If c>ret Then ret=c;bucket=x
		Next
		Return bucket
	End Method
	Rem
	bbdoc: Get the contents of the bucket at some index as objects
	returns: A TList containing the objects of the nodes in a bucket
	End Rem
	Method bucket:TList(index%)
		Local r:TList=CreateList()
		If Not data[index] Return r
		For Local n:HashNode=EachIn data[index]
			r.addlast n.data
		Next
		Return r
	End Method
	Rem
	bbdoc: Get the size of the hash table
	returns: The size of the hash table
	End Rem
	Method size%()
		Return data.length
	End Method
	Rem
	bbdoc: Insert a node into the hash table
	End Rem
	Method insertnode(n:HashNode)
		Local i%=Abs(hashfunc(n.key) Mod data.length)
		If Not data[i] Then data[i]=CreateList()
		data[i].addfirst n
	End Method
	Rem
	bbdoc: Insert a key and value pair into the hash table
	returns: The new node
	End Rem
	Method insert:HashNode(key$,value:Object)
		Local n:HashNode=HashNode.Create(key,value)
		insertnode n
		Return n
	End Method
	Rem
	bbdoc: Find the list link of either a specific node or the first node with a given key
	returns: The TLink for the node
	about: @bringtofront determines whether the found link should also be put to the front of the bucket
	End Rem
	Method findnodelink:TLink(key$,n:HashNode=Null,bringtofront%=-1)
		'if n is null then return the first node with that key
		Local l:TList=data[Abs(hashfunc(key) Mod data.length)]
		If Not l Return Null
		Local link:TLink=l._head._succ
		While link<>link._value
			If n
				If HashNode(link._value)=n
					If bringtofront=1 Or (bringtofront=-1 And autobringtofront) Then movelinktofront link,l
					Return link
				EndIf
			Else
				If HashNode(link._value).key=key 
					If bringtofront=1 Or (bringtofront=-1 And autobringtofront) Then movelinktofront link,l
					Return link
				EndIf
			EndIf
			link=link._succ
		Wend
		Return Null
	End Method
	Rem
	bbdoc: Find the list links of all the nodes with a given key
	returns: A list of TLinks for the nodes
	End Rem
	Method findallnodelinks:TList(key$)
		Local ret:TList=CreateList()
		Local l:TList=data[Abs(hashfunc(key) Mod data.length)]
		If Not l Return ret
		Local link:TLink=l._head._succ
		While link<>link._value
			If HashNode(link._value).key=key Then ret.addlast link
			link=link._succ
		Wend
		Return ret
	End Method
	Rem
	bbdoc: Get the first node with a key
	returns: The first node with a key
	End Rem
	Method findnode:HashNode(key$,bringtofront%=-1)
		Local l:TList=data[Abs(hashfunc(key) Mod data.length)]
		If Not l Return Null
		Local link:TLink=findnodelink(key,Null,bringtofront)
		If link Return HashNode(link._value)
		Return Null
	End Method
	Rem
	bbdoc: Get all the nodes with a key
	returns: A TList of all the nodes with a key
	End Rem
	Method findallnodes:TList(key$)
		Local ret:TList=CreateList()
		Local l:TList=data[Abs(hashfunc(key) Mod data.length)]
		If Not l Return ret
		For Local n:HashNode=EachIn l
			If n.key=key ret.addlast n
		Next
		Return ret
	End Method
	Rem
	bbdoc: Get the first object associated with a key
	returns: The first object associated with a key
	End Rem
	Method find:Object(key$,bringtofront%=-1)
		Local n:HashNode=findnode(key,bringtofront)
		If Not n Return Null
		Return n.data
	End Method
	Rem
	bbdoc: Get all the objects associated with a key
	returns: A TList of all the objects associated with a key
	End Rem
	Method findall:TList(key$)
		Local ret:TList=CreateList()
		Local l:TList=data[Abs(hashfunc(key) Mod data.length)]
		If Not l Return ret
		For Local n:HashNode=EachIn l
			If n.key=key ret.addlast n.data
		Next
		Return ret
	End Method
	Rem
	bbdoc: Remove a specific node from the hash table
	returns: 1 if the node was successfully removed, 0 if something went wrong
	End Rem
	Method removenode%(n:HashNode)
		Local l:TLink=findnodelink(n.key,n,False)
		If Not l Then Return 0
		l.remove;Return 1
	End Method
	Rem
	bbdoc: Remove the first node with a given key
	returns: The node that was removed
	End Rem
	Method remove:HashNode(key$)
		Local l:TLink=findnodelink(key,Null,False)
		If Not l Then Return Null
		Local v:HashNode=HashNode(l._value)
		l.remove;Return v
	End Method
	Rem
	bbdoc: Remove all the nodes with a given key
	returns: The number of nodes that were removed
	End Rem
	Method removeall%(key$)
		Local list:TList=findallnodelinks(key),n%=0
		For Local l:TLink=EachIn list
			l.remove;n:+1
		Next
		Return n
	End Method
	Rem
	bbdoc: Remove all the nodes with a given key
	returns: A TList containing the nodes that were removed
	End Rem
	Method removeallnodes:TList(key$)
		Local list:TList=findallnodelinks(key),n%=0,ret:TList=CreateList()
		For Local l:TLink=EachIn list
			ret.addlast HashNode(l._value)
			l.remove;n:+1
		Next
		Return ret
	End Method
	Rem
	bbdoc: Create a new hash table
	returns: A new hash table
	End Rem
	Function Create:HashTable(size%,hashfunc%(key$)=dbgHashKey)
		Local h:HashTable=New HashTable
		h.setsize size
		h.hashfunc=hashfunc
		Return h
	End Function
	Rem
	bbdoc: Method implements #EachIn support
	returns: An iterator object
	about: Iterates through the objects in the hash table
	End Rem
	Method ObjectEnumerator:HashEnum()
		Return HashEnum.Create(data,0)
	End Method
	Rem
	bbdoc: Method implements #EachIn support
	returns: An iterator object
	about: Iterates through the keys in the hash table (considers duplicates as they crop up)
	End Rem
	Method KeyEnumerator:HashEnum()
		Return HashEnum.Create(data,1)
	End Method
	Rem
	bbdoc: Method implements #EachIn support
	returns: An iterator object
	about: Iterates through the nodes in the hash table
	End Rem
	Method NodeEnumerator:HashNodeEnum()
		Return HashNodeEnum.Create(data)
	End Method
End Type
	Rem
	bbdoc: A hash table node object
	End Rem
Type HashNode
	Field key$,data:Object
	Rem
	bbdoc: Create a new hash table node
	returns: A new hash table node
	End Rem
	Function Create:HashNode(key$,value:Object)
		Local n:HashNode=New HashNode
		n.key=key;n.data=value
		Return n
	End Function
End Type

Type HashEnum
	Field array:TList[],index%=0,link:TLink,mode%=0
	Function Create:HashEnum(array:TList[],mode%=0)
		Local h:HashEnum=New HashEnum
		h.array=array
		h.gotofirst
		h.mode=mode
		Return h
	End Function
	Method gotofirst()
		While Not link
			If array[index]
				If array[index]._head._succ<>array[index]._head Then 
					link=array[index]._head._succ
					Exit
				EndIf
			EndIf
			index:+1
		Wend
	End Method
	Method HasNext%()
		If link<>link._value Return 1
		While array.length-index>1
			index:+1
			If array[index] Then
				If array[index]._head._succ=array[index]._head Then Continue
				link=array[index]._head._succ
				Return 1
			EndIf
		Wend
		Return 0
	End Method
	Method NextObject:Object()
		Local ret:Object
		If mode=1 ret=HashNode(link._value).key Else ret=HashNode(link._value).data
		link=link._succ
		Return ret
	End Method
	Method ObjectEnumerator:HashEnum()
		Return Self
	End Method
End Type

Type HashNodeEnum Extends HashEnum
	Function Create:HashNodeEnum(array:TList[],mode%=0)
		Local h:HashNodeEnum=New HashNodeEnum
		h.array=array
		h.gotofirst
		h.mode=mode
		Return h
	End Function
	Method NextObject:Object()
		Local ret:Object=HashNode(link._value)
		link=link._succ
		Return ret
	End Method
End Type



' Thanks to Robert Jenkins for the original function
' Read his paper http://burtleburtle.net/bob/hash/evahash.html
' Used to give a clear stack trace
' credit to http://www.blitzbasic.com/codearcs/codearcs.php?code=1717 for writing the algorithm for blitzmax
	Rem
	bbdoc: Native hash function
	returns: A hash
	End Rem
Function dbgHashKey:Int(s:String)
	Local a%,b%,c%,l%,p@ Ptr,k@ Ptr
	p=s.ToCString();k=p;l=s.Length;a=$9e3779b9;b=a;c=52867
	While l>=12
		a:+k[0]+(k[1] Shl 8)+(k[2] Shl 16)+(k[3] Shl 24)
		b:+k[4]+(k[5] Shl 8)+(k[6] Shl 16)+(k[7] Shl 24)
		c:+k[8]+(k[9] Shl 8)+(k[10] Shl 16)+(k[11] Shl 24)
		a=a-b;a=a-c;a=a~(c Shr 13)
		b=b-c;b=b-a;b=b~(a Shl 8)
		c=c-a;c=c-b;c=c~(b Shr 13)
		a=a-b;a=a-c;a=a~(c Shr 12)
		b=b-c;b=b-a;b=b~(a Shl 16)
		c=c-a;c=c-b;c=c~(b Shr 5)
		a=a-b;a=a-c;a=a~(c Shr 3)
		b=b-c;b=b-a;b=b~(a Shl 10)
		c=c-a;c=c-b;c=c~(b Shr 15)
		k:+12;l:-12
	Wend
	c:+s.Length
	Select l
		Case 11 c=c+(k[10] Shl 24)
		Case 10 c=c+(k[9] Shl 16)
		Case 9  c=c+(k[8] Shl 8)
		Case 8  b=b+(k[7] Shl 24);
		Case 7  b=b+(k[6] Shl 16);
		Case 6  b=b+(k[5] Shl 8);
		Case 5  b=b+k[4];
		Case 4  a=a+(k[3] Shl 24);
		Case 3  a=a+(k[2] Shl 16);
		Case 2  a=a+(k[1] Shl 8)
		Case 1  a=a+k[0];
	End Select
	a=a-b;a=a-c;a=a~(c Shr 13)
	b=b-c;b=b-a;b=b~(a Shl 8)
	c=c-a;c=c-b;c=c~(b Shr 13)
	a=a-b;a=a-c;a=a~(c Shr 12)
	b=b-c;b=b-a;b=b~(a Shl 16)
	c=c-a;c=c-b;c=c~(b Shr 5)
	a=a-b;a=a-c;a=a~(c Shr 3)
	b=b-c;b=b-a;b=b~(a Shl 10)
	c=c-a;c=c-b;c=c~(b Shr 15)
	MemFree(p);Return c
End Function


Private

Function movelinktofront(link:TLink,list:TList)
	If link=list.firstlink() Then Return
	link._succ._pred=link._pred
	link._pred._succ=link._succ
	link._pred=list._head
	link._succ=list._head._succ
	link._succ._pred=link
	list._head._succ=link
End Function




